def axi_connectN(field, io_prefix, wire_prefix):
    for i in field:
        print(".{}_{}\t".format(io_prefix, i), end="")
        print("({", end="")
        for j in wire_prefix:
            if j == wire_prefix[-1]:
                print("{}_{}".format(j, i), end="")
            else:
                print("{}_{},".format(j, i), end="\t")
        print("}),")



from axi_connectN import axi_connectN
from axi_parameter import LEN_WIDTH, SIZE_WIDTH, BURST_WIDTH, LOCK_WIDTH, CACHE_WIDTH, PROT_WIDTH, QOS_WIDTH, REGION_WIDTH, USER_WIDTH
# FIELD    WIDTH  axi_crossbar_slave   axi_crossbar_master     axi_ram
# awid      0-32        YES                      NO                  YES      : Write address ID
# awaddr   12-64        YES                      YES                 YES      : Write address
# awlen      8          YES                      YES                 YES      : Write burst length
# awsize     3          YES                      YES                 YES      : Write burst size
# awburst    2          YES                      YES                 YES      : Write burst type
# awlock     1          YES                      YES                 NO       : Write locking
# awcache    4          YES                      YES                 NO       : Write cache handling
# awprot     3          YES                      YES                 NO       : Write protection level
# awqos      4          YES                      YES                 NO       : Write QoS setting
# awregion   4          NO                       YES                 YES      : Write region
# awuser     1          NO                       NO                  NO       : Write user sideband signal
# awvalid    1          YES                      YES                 YES      : Write address valid
# awready    1          YES                      YES                 YES      : Write address ready (from slave)


class axi_aw_channel:
    axi_aw_field = ["awid", "awaddr", "awlen", "awsize", "awburst", "awlock",
                    "awcache", "awprot", "awqos", "awregion", "awvalid", "awready"]

    axi_aw_width = {"awid": 4, "awaddr": 32, "awlen": LEN_WIDTH, "awsize": SIZE_WIDTH, "awburst": BURST_WIDTH, "awlock": LOCK_WIDTH,
                    "awcache": CACHE_WIDTH, "awprot": PROT_WIDTH, "awqos": QOS_WIDTH, "awregion": REGION_WIDTH, "awvalid": 1, "awready": 1}

    def __init__(self, PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH):
        self.PREFIX = PREFIX
        self.axi_aw_width["awaddr"] = ADDR_WIDTH
        self.axi_aw_width["awid"] = ID_WIDTH

    def def_wire(self):
        self_aw_field = self.axi_aw_field.copy()
        self_aw_width = self.axi_aw_width.copy()
        assert(len(self_aw_field) == len(self_aw_width))
        assert(len(self_aw_width) == 12)
        for w, f in zip(self_aw_width.values(), self_aw_field):
            print("wire[{} - 1:0] {}_{};".format(w, self.PREFIX, f))

    def def_connectN(self, io_prefix, wire_prefix, direction):
        assert((direction == "CROSSBAR_MASTER") | (
            direction == "CROSSBAR_SLAVE") | (direction == "BRAM_PORT") | (direction == "NATIVE_TO_AXI") | (direction == "AXI_TO_NATIVE"))

        self_aw_field = self.axi_aw_field.copy()
        if direction == "CROSSBAR_MASTER":
            # self_aw_field.remove("awuser")
            self_aw_field.remove("awid")
            assert(len(self_aw_field) == 11)
        elif direction == "CROSSBAR_SLAVE":
            self_aw_field.remove("awregion")
            # self_aw_field.remove("awuser")
            assert(len(self_aw_field) == 11)
        elif direction == "BRAM_PORT":
            self_aw_field.remove("awregion")
            # self_aw_field.remove("awuser")
            self_aw_field.remove("awlock")
            self_aw_field.remove("awcache")
            self_aw_field.remove("awqos")
            self_aw_field.remove("awprot")
            assert(len(self_aw_field) == 7)
        elif direction == "NATIVE_TO_AXI":
            # self_aw_field.remove("awuser")
            assert(len(self_aw_field) == 12)
        elif direction == "AXI_TO_NATIVE":
            self_aw_field.remove("awregion")
            # self_aw_field.remove("awuser")
            assert(len(self_aw_field) == 11)
        axi_connectN(self_aw_field, io_prefix, wire_prefix)

    def def_output(self, direction="MASTER"):
        assert((direction == "MASTER") | (direction == "SLAVE"))

        self_aw_field = self.axi_aw_field.copy()
        self_aw_width = self.axi_aw_width.copy()
        if direction == "MASTER":
            dir1 = "output"
            dir2 = "input"

        else:
            dir1 = "input"
            dir2 = "output"
        self_aw_dir = [dir1] * (len(self_aw_field) - 1) + [dir2]
        assert(len(self_aw_field) == len(self_aw_width)
               & len(self_aw_field) == len(self_aw_dir))
        for d, w, f in zip(self_aw_dir, self_aw_width.values(), self_aw_field):
            print("{} wire[{} - 1:0] {}_{},".format(d, w, self.PREFIX, f))



from axi_connectN import axi_connectN
from axi_parameter import LEN_WIDTH, SIZE_WIDTH, BURST_WIDTH, LOCK_WIDTH, CACHE_WIDTH, PROT_WIDTH, QOS_WIDTH, REGION_WIDTH, USER_WIDTH


class axi_ar_channel:
    axi_ar_field = ["arid", "araddr", "arlen", "arsize",  "arburst",
                    "arlock", "arcache", "arprot", "arqos", "arregion",  "arvalid", "arready"]
    axi_ar_width = {"arid": 4, "araddr": 32, "arlen": LEN_WIDTH, "arsize": SIZE_WIDTH, "arburst": BURST_WIDTH,
                    "arlock": LOCK_WIDTH, "arcache": CACHE_WIDTH, "arprot": PROT_WIDTH, "arqos": QOS_WIDTH, "arregion": REGION_WIDTH,  "arvalid": 1, "arready": 1}

    def __init__(self, PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH):
        self.PREFIX = PREFIX
        self.axi_ar_width["araddr"] = ADDR_WIDTH
        self.axi_ar_width["arid"] = ID_WIDTH

    def def_wire(self):
        self_ar_field = self.axi_ar_field.copy()
        self_ar_width = self.axi_ar_width.copy()
        assert(len(self_ar_field) == len(self_ar_width))
        assert(len(self_ar_width) == 12)
        for w, f in zip(self_ar_width.values(), self_ar_field):
            print("wire[{} - 1:0] {}_{};".format(w, self.PREFIX, f))

    def def_connectN(self, io_prefix, wire_prefix, direction):
        assert((direction == "CROSSBAR_MASTER") | (
            direction == "CROSSBAR_SLAVE") | (direction == "BRAM_PORT") | (direction == "NATIVE_TO_AXI") | (direction == "AXI_TO_NATIVE"))

        self_ar_field = self.axi_ar_field.copy()
        if direction == "CROSSBAR_MASTER":
            self_ar_field.remove("arid")
            # self_ar_field.remove("aruser")
            assert(len(self_ar_field) == 11)
        elif direction == "CROSSBAR_SLAVE":
            self_ar_field.remove("arregion")
            # self_ar_field.remove("aruser")
            assert(len(self_ar_field) == 11)
        elif direction == "BRAM_PORT":
            self_ar_field.remove("arregion")
            # self_ar_field.remove("aruser")
            self_ar_field.remove("arlock")
            self_ar_field.remove("arcache")
            self_ar_field.remove("arqos")
            self_ar_field.remove("arprot")
            assert(len(self_ar_field) == 7)
        elif direction == "NATIVE_TO_AXI":
            assert(len(self_ar_field) == 12)
        elif direction == "AXI_TO_NATIVE":
            self_ar_field.remove("arregion")
            # self_ar_field.remove("aruser")
            assert(len(self_ar_field) == 11)
        axi_connectN(self_ar_field, io_prefix, wire_prefix)

    def def_output(self, direction="MASTER"):
        assert((direction == "MASTER") | (direction == "SLAVE"))
        if direction == "MASTER":
            dir1 = "output"
            dir2 = "input"
        else:
            dir1 = "input"
            dir2 = "output"
        self_ar_field = self.axi_ar_field.copy()
        self_ar_width = self.axi_ar_width.copy()
        self_ar_dir = [dir1] * (len(self_ar_field) - 1) + [dir2]
        assert(len(self_ar_field) == len(self_ar_width)
               & len(self_ar_field) == len(self_ar_dir))
        for d, w, f in zip(self_ar_dir, self_ar_width.values(), self_ar_field):
            print("{} wire[{} - 1:0] {}_{},".format(d, w, self.PREFIX, f))



from axi_connectN import axi_connectN
from axi_parameter import USER_WIDTH, RESP_WIDTH
# FIELD    axi_crossbar_slave   axi_crossbar_master     axi_ram
# bid         YES                      NO                  YES      : Write response ID
# bresp       YES                      YES                 YES      : Write response
# buser       NO                       NO                  NO       : Write response user sideband signal
# bvalid      YES                      YES                 YES      : Write response valid
# bready      YES                      YES                 YES      : Write response ready (from master)


class axi_b_channel:
    axi_b_field = ["bid", "bresp",  "bvalid", "bready"]
    axi_b_width = {"bid": 4, "bresp": RESP_WIDTH, "bvalid": 1, "bready": 1}

    def __init__(self, PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH):
        self.PREFIX = PREFIX
        self.axi_b_width["bid"] = ID_WIDTH

    def def_wire(self):
        self_b_field = self.axi_b_field.copy()
        self_b_width = self.axi_b_width.copy()
        assert(len(self_b_field) == len(self_b_width))
        assert(len(self_b_width) == 4)
        for w, f in zip(self_b_width.values(), self_b_field):
            print("wire[{} - 1:0] {}_{};".format(w, self.PREFIX, f))

    def def_connectN(self, io_prefix, wire_prefix, direction):
        assert((direction == "CROSSBAR_MASTER") | (
            direction == "CROSSBAR_SLAVE") | (direction == "BRAM_PORT") | (direction == "NATIVE_TO_AXI") | (direction == "AXI_TO_NATIVE"))
        self_b_field = self.axi_b_field.copy()
        if direction == "CROSSBAR_MASTER":
            self_b_field.remove("bid")
            # self_b_field.remove("buser")
            assert(len(self_b_field) == 3)
        elif direction == "CROSSBAR_SLAVE":
            # self_b_field.remove("buser")
            assert(len(self_b_field) == 4)
        elif direction == "BRAM_PORT":
            # self_b_field.remove("buser")
            assert(len(self_b_field) == 4)
        elif direction == "NATIVE_TO_AXI":
            assert(len(self_b_field) == 4)
        elif direction == "AXI_TO_NATIVE":
            assert(len(self_b_field) == 4)
        axi_connectN(self_b_field, io_prefix, wire_prefix)

    def def_output(self, direction="MASTER"):
        assert((direction == "MASTER") | (direction == "SLAVE"))
        if direction == "MASTER":
            dir1 = "input"
            dir2 = "output"
        else:
            dir1 = "output"
            dir2 = "input"

        self_b_field = self.axi_b_field.copy()
        self_b_width = self.axi_b_width.copy()
        self_b_dir = [dir1] * (len(self_b_field) - 1) + [dir2]
        assert(len(self_b_field) == len(self_b_width)
               & len(self_b_field) == len(self_b_dir))
        for d, w, f in zip(self_b_dir, self_b_width.values(), self_b_field):
            print("{} wire[{} - 1:0] {}_{},".format(d, w, self.PREFIX, f))



from axi_aw_channel import axi_aw_channel
from axi_w_channel import axi_w_channel
from axi_b_channel import axi_b_channel
from axi_ar_channel import axi_ar_channel
from axi_r_channel import axi_r_channel


# ADDR_WIDTH           : width of awaddr and araddr signals
# DATA_WIDTH           : width of wdata and rdata signals
# STRB_WIDTH           : width of wstrb signal
# ID_WIDTH             : width of *id signals
# AWUSER_ENABLE        : enable awuser signal
# AWUSER_WIDTH         : width of awuser signal
# WUSER_ENABLE         : enable wuser signal
# WUSER_WIDTH          : width of wuser signal
# BUSER_ENABLE         : enable buser signal
# BUSER_WIDTH          : width of buser signal
# ARUSER_ENABLE        : enable aruser signal
# ARUSER_WIDTH         : width of aruser signal
# RUSER_ENABLE         : enable ruser signal
# RUSER_WIDTH          : width of ruser signal

class axi():
    def __init__(self, PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH):
        self.axi_ar = axi_ar_channel(PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH)
        self.axi_aw = axi_aw_channel(PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH)
        self.axi_r = axi_r_channel(PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH)
        self.axi_w = axi_w_channel(PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH)
        self.axi_b = axi_b_channel(PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH)

    def def_wire(self):
        self.axi_ar.def_wire()
        self.axi_r.def_wire()
        self.axi_aw.def_wire()
        self.axi_w.def_wire()
        self.axi_b.def_wire()

    def def_connectN(self, io_prefix, wire_prefix, direction="CROSSBAR_SLAVE"):
        self.axi_ar.def_connectN(io_prefix, wire_prefix, direction)
        self.axi_aw.def_connectN(io_prefix, wire_prefix, direction)
        self.axi_r.def_connectN(io_prefix, wire_prefix, direction)
        self.axi_w.def_connectN(io_prefix, wire_prefix, direction)
        self.axi_b.def_connectN(io_prefix, wire_prefix, direction)

    def def_output(self, direction="MASTER"):
        self.axi_ar.def_output(direction)
        self.axi_aw.def_output(direction)
        self.axi_r.def_output(direction)
        self.axi_w.def_output(direction)
        self.axi_b.def_output(direction)



from setuptools import setup

setup(
    name='verilog_axi4',
    version='v0.12',
    description='A module for generating verilog axi4 code',
    py_modules=['verilog_axi4', 'axi_ar_channel', 'axi_aw_channel', 'axi_b_channel',
                'axi_r_channel', 'axi_w_channel', 'axi_parameter', 'axi_connectN'],
    author='likmi',
    author_email='likmi@qq.com',
    license='MIT'
)



from axi_connectN import axi_connectN
from axi_parameter import STRB_WIDTH, USER_WIDTH


class axi_w_channel:
    axi_w_field = ["wid", "wdata", "wstrb",
                   "wlast", "wvalid", "wready"]
    axi_w_width = {"wid": 1, "wdata": 64, "wstrb": STRB_WIDTH,
                   "wlast": 1, "wvalid": 1, "wready": 1}

    def __init__(self, PREFIX, ADDR_WIDTH, DATA_WIDTH, ID_WIDTH):
        self.PREFIX = PREFIX
        self.axi_w_width["wid"] = ID_WIDTH
        self.axi_w_width["wdata"] = DATA_WIDTH
        self.axi_w_width["wstrb"] = int(DATA_WIDTH/8)

    def def_wire(self):
        self_w_field = self.axi_w_field.copy()
        self_w_width = self.axi_w_width.copy()

        assert(len(self_w_field) == len(self_w_width))

        for w, f in zip(self_w_width.values(), self_w_field):
            print("wire[{} - 1:0] {}_{};".format(w, self.PREFIX, f))

    def def_connectN(self, io_prefix, wire_prefix, direction):
        assert((direction == "CROSSBAR_MASTER") | (
            direction == "CROSSBAR_SLAVE") | (direction == "BRAM_PORT") | (direction == "NATIVE_TO_AXI") | (direction == "AXI_TO_NATIVE"))

        self_w_field = self.axi_w_field.copy()
        if direction == "CROSSBAR_MASTER":
            self_w_field.remove("wid")
            # self_w_field.remove("wuser")
            assert(len(self_w_field) == 5)
        elif direction == "CROSSBAR_SLAVE":
            self_w_field.remove("wid")
            # self_w_field.remove("wuser")
            assert(len(self_w_field) == 5)
        elif direction == "BRAM_PORT":
            self_w_field.remove("wid")
            # self_w_field.remove("wuser")
            assert(len(self_w_field) == 5)
        elif direction == "NATIVE_TO_AXI":
            self_w_field.remove("wid")
            assert(len(self_w_field) == 5)
        elif direction == "AXI_TO_NATIVE":
            self_w_field.remove("wid")
            # self_w_field.remove("wuser")
            assert(len(self_w_field) == 5)
        axi_connectN(self_w_field, io_prefix, wire_prefix)

    def def_output(self, direction="MASTER"):
        assert((direction == "MASTER") | (direction == "SLAVE"))
        if direction == "MASTER":
            dir1 = "output"
            dir2 = "input"
        else:
            dir1 = "input"
            dir2 = "output"
        self_w_field = self.axi_w_field.copy()
        self_w_width = self.axi_w_width.copy()
        self_w_dir = [dir1] * (len(self_w_field) - 1) + [dir2]
        assert(len(self_w_field) == len(self_w_width)
               & len(self_w_field) == len(self_w_dir))
        for d, w, f in zip(self_w_dir, self.axi_w_width.values(), self_w_field):
            print("{} wire[{} - 1:0] {}_{},".format(d, w, self.PREFIX, f))
